;###################################################################################
;#                                                                                 #
;#                              LCD HD44780 Lib                                    #
;#                             ------------------                                  #
;#                                                                                 #
;# Microcontroller: egal                                                           #
;# Quarz:           egal                                                           #
;#                                                                                 #
;# Autor: Denis B.	                                                           #
;#        www.elektronik4freaks.de.tk                                              #
;#                                                                                 #
;# Version: 1.4                                                                    #
;#                                                                                 #
;# Register: * r17                                                                 #
;#           * r18                                                                 #
;#           * r19                                                                 #
;#                                                                                 #
;# Stack: wird ben�tigt                                                            #
;#                                                                                 #
;# Ports: LCD_Port ( mind. 5 Portpins )                                            #
;#                                                                                 #
;###################################################################################


;---[ Definitionen der Datenleitungen ]---------------------------------------------

.equ db0 = 0
.equ db1 = 1
.equ db2 = 2 
.equ db3 = 3
.equ db4 = 4
.equ db5 = 5
.equ db6 = 6
.equ db7 = 7

.equ rs = 2
.equ en = 3

.equ lcd_port = porta
.equ lcd_ddr  = ddra         	; LCD-Display


;---[ Initialisierung ]-------------------------------------------------------------

lcd_init:

 push r17			; Rettung der verw. Register
 push r18
 push r19 
  
 ldi r19 , 10         
 
 poweron:                    	; Wartet 10x 5ms nach dem Einschalten von Vcc                     
                
  rcall waitms5              
  dec r19
  brne poweron

 ldi r17, (1<<db4)|(1<<db5)  	; Datenleitung 4 und 5 setzen und 3mal senden
 
 out lcd_port, r17           	; 1mal
 rcall toogleen

 rcall waitms5               	; und auf das lahme LCD warten

 out lcd_port, r17           	; 2mal
 rcall toogleen

 rcall waitms5               	; wieder warten

 out lcd_port, r17           	; 3mal
 rcall toogleen

 rcall waitms5               	; und nochmals warten

 ldi r17, (1<<db5)           	; 4-Bit Modus aktivieren
 out lcd_port,r17
 rcall toogleen

 rcall waitms5               	; und schon wieder 5ms warten

 ldi r17, (1<<db5)|(1<<db3)  	; 5x8-Dots und 2-Lines
 rcall lcd_com

 ldi r17, 0b00001110         	; Display  On
 rcall lcd_com

 rcall lcd_cls               	; Display l�schen

 ldi r17, (1<<db1)|(1<<db2)  	; Entrymode Set
 rcall lcd_com

 pop r19			; Wiederherstellung der verw. Register
 pop r18
 pop r17

ret


;---[ LCD-Daten senden ]------------------------------------------------------------

lcd_out:

 push r17			; Rettung der verw. Register
 push r18
 
 mov r18, r17                	; Databyte kopieren
 

 andi r17, 0b11110000        	; es d�rfen ja nur die Pin5 - 7 gesetzt werden
 
 out lcd_port, r17          	; und ab ins LCD damit
 sbi lcd_port, rs            	; und die RS Leitung setzen
 rcall toogleen 
        
 cbi lcd_port, rs            	; und die RS Leitung r�cksetzen

 swap r18                    	; Nippeltausch ( Hightbits zuerst senden )
 andi r18, 0b11110000        	; so und jetzt die 4 Lowbits f�rn Port zuschneiden
 
 out lcd_port, r18           	; und ab ins LCD
 sbi lcd_port, rs            	; RS Leitung setzen 
 rcall toogleen 
 
 cbi lcd_port, rs            	; RS Leitung r�cksetzen

 rcall waitus50              	; und wieder 50�S warten 
     
 pop r18			; Wiederherstellung der verw. Register
 pop r17

ret


;---[ LCD-Befehle senden ]----------------------------------------------------------

lcd_com:
 
 push r17			; Rettung der verw. Register			
 push r18

 mov r18, r17			; Befehlsbyte kopieren
 

 andi r17, 0b11110000        	; es d�rfen ja nur die Pin5 - 7 gesetzt werden
 
 out lcd_port, r17           	; und ab ins LCD damit
 rcall toogleen 
 
 swap r18                    	; Nippeltausch ( Hightbits zuerst senden )          
 andi r18, 0b11110000        	; so und jetzt die 4 Lowbits f�rn Port zuschneiden
 
 out lcd_port, r18           	; und ab ins LCD 
 rcall toogleen 
 
 rcall waitus50              	; und wieder 50�S warten     

 pop r18			; Wiederherstellung der verw. Register
 pop r17

ret


;---[ Display l�schen ]-------------------------------------------------------------

lcd_cls:

 push r17			; Rettung der verw. Register

 ldi r17, (1<<db0)		; L�schbefehl ans LCD senden
 rcall lcd_com

 rcall waitms5			; Warten bis Display gel�scht

 pop r17			; Wiederherstellung der verw. Register

ret


;---[ Enable Set/Reset ]------------------------------------------------------------

toogleen:

 sbi lcd_port,en             	; Enable auf High setzen
 nop                         	; 1�Sec Warten
 nop
 nop
 nop
 cbi lcd_port,en             	; Enable au Low setzen

ret


;---[ 5mS warten ]------------------------------------------------------------------

waitms5:                     	; Ganz genau 5ms warten (19990 + 10 Clocks bei 4MHz)

           ldi  r17, $3C
 WGLP0:    ldi  r18, $DD
 WGLP1:    dec  r18
           brne WGLP1
           dec  r17
           brne WGLP0
           ldi  r17, $03
 WGLP2:    dec  r17
           brne WGLP2
           nop

ret


;---[ 190�S warten ]----------------------------------------------------------------

waitus50:                    	; Genau 100�S warten

           ldi  r17, $FC
 WGLP3:    dec  r17
           brne WGLP3
           nop

ret
